@@ -9,10 +9,13 @@ use vipers::prelude::*;
 mod account_validators;
 mod macros;
 mod redeem_cpi;
+mod state;
+
+pub use state::*;
 
 declare_id!("QRDxhMw1P2NEfiw5mYXG79bwfgHTdasY2xNP76XSea9");
 
-#[cfg(not(feature = "cpi"))]
+#[cfg(not(feature = "no-entrypoint"))]
 solana_security_txt::security_txt! {
     name: "Quarry Redeemer",
     project_url: "https://quarry.so",
@@ -34,7 +37,7 @@ pub mod quarry_redeemer {
         let redeemer = &mut ctx.accounts.redeemer;
         redeemer.iou_mint = ctx.accounts.iou_mint.key();
         redeemer.redemption_mint = ctx.accounts.redemption_mint.key();
-        redeemer.bump = *unwrap_int!(ctx.bumps.get("redeemer"));
+        redeemer.bump = unwrap_bump!(ctx, "redeemer");
 
         redeemer.total_tokens_redeemed = 0;
         Ok(())
@@ -82,25 +85,6 @@ pub mod quarry_redeemer {
 // Accounts
 // --------------------------------
 
-/// Redeemer state
-#[account]
-#[derive(Copy, Default, Debug, PartialEq, Eq, PartialOrd, Ord)]
-pub struct Redeemer {
-    /// [Mint] of the IOU token.
-    pub iou_mint: Pubkey,
-    /// [Mint] of the token to redeem.
-    pub redemption_mint: Pubkey,
-    /// Bump seed.
-    pub bump: u8,
-
-    /// Lifetime number of IOU tokens redeemed for redemption tokens.
-    pub total_tokens_redeemed: u64,
-}
-
-impl Redeemer {
-    pub const LEN: usize = 32 + 32 + 1 + 8;
-}
-
 // --------------------------------
 // Instructions
 // --------------------------------
@@ -150,7 +134,7 @@ pub struct RedeemTokens<'info> {
     #[account(mut)]
     pub redemption_vault: Account<'info, TokenAccount>,
     /// Destination of the IOU tokens.
-    #[account(mut)]
+    #[account(mut, constraint = redemption_destination.key() != redemption_vault.key())]
     pub redemption_destination: Account<'info, TokenAccount>,
 
     /// The spl_token program corresponding to [Token].
